from odoo import api, models
from odoo import tools
from odoo.addons.saas_worker.util import version_gte

if version_gte('15.0'):

    class IrAsset(models.Model):

        _inherit = 'ir.asset'
        _name = 'ir.asset'

        @api.model
        @tools.ormcache('addons_tuple')
        def _topological_sort(self, addons_tuple):
            """
            The OWL asset manager ensure to order modules dependancies topologically,
            based on the modules manifests, before computing heritage between assets.

            The problem in our SaaS, is that we have heritage from asset declared in
            web_enterprise for database post v15.0, but as `saas_trial` needs to remain
            retro-compatible, we can't set `web_enterprise` as a dependancy in its
            manifest because `web_enterprise` didn't existed in v8.0.

            Hence the reason for the ugly hack below, it ensures that `saas_trial` will be
            placed after `web_enterprise`, to prevent the client the following crash
            when building the assets:

            `ValueError: Module web_enterprise not loaded or inexistent, or templates of addon being loaded (saas_trial) are misordered`

            It shouldn't be an issue anyway because `web_enterprise` is installed
            on all our SaaS databases.
            """

            sorted_list = super(IrAsset, self)._topological_sort(addons_tuple)

            # Reorder only when really needed
            if {'saas_trial', 'web_enterprise'} <= set(sorted_list) \
                and sorted_list.index('saas_trial') < sorted_list.index('web_enterprise'):

                sorted_list.insert(
                    sorted_list.index('saas_trial'),
                    sorted_list.pop(sorted_list.index('web_enterprise'))
                )

            return sorted_list
